<?php
/*
 * Iso8601Converter - Convert time and period inputs to standard format
 * Copyright 2011 Kappa Computer Solutions, LLC and Brian Katzung
 *
 * Version 1.2, 2011-08-05 Brian Katzung
 * Version 1.1, 2011-07-24 Brian Katzung
 *
 * License: MIT http://www.opensource.org/licenses/mit-license.php
 */

class Iso8601Converter {

    protected static $defaultDateOrder = 'auto';
    protected $dateOrder;
    protected $isValid = false;
    protected $year, $month, $day, $hour, $minute;

    # Constructor
    function __construct ($str = null) {
	$this->dateOrder(self::defaultDateOrder());
	if (isset($str)) {
	    $this->parse($str);
	}
    }

    # Set class default date order
    function defaultDateOrder ($order = null) {
	if (self::isValidDateOrder($order)) {
	    self::$defaultDateOrder = $order;
	}
	return self::$defaultDateOrder;
    }

    # Set instance date order
    function dateOrder ($order = null) {
	if (self::isValidDateOrder($order)) {
	    $this->dateOrder = $order;
	}
	return $this->dateOrder;
    }

    # Is this a valid date order?
    function isValidDateOrder ($order) {
	switch ($order) {
	case 'auto': case 'dmy': case 'mdy': case 'ymd': return true;
	default: return false;
	}
    }

    # Class method to return a period as a string
    static function periodToString ($years, $months, $days,
      $hours = 0, $minutes = 0, $zero = 'P0D') {
	if (!$years && !$months && !$days && !$hours && !$minutes) {
	    return $zero;
	}

	$period = 'P';
	if ($years) { $period .= "{$years}Y"; }
	if ($months) { $period .= "{$months}M"; }
	if ($days) { $period .= "{$days}D"; }
	if ($hours || $minutes) { $period .= 'T'; }
	if ($hours) { $period .= "{$hours}H"; }
	if ($minutes) { $period .= "{$minutes}M"; }
	return $period;
    }

    # Has instance parsed a valid input?
    function isValid () { return $this->isValid; }

    # Instance method to parse time string
    function parse ($str) {
	if ($this->parseIso8601($str)) {
	    return true;
	}

	$time_re = '/(\d+)([\D\S])(\d+)[\D\S](\d+)(?:[T\s]+(\d+)(?::(\d+))?\s*([AP])?)?/i';
	# parts, year, month, day, hour, minute, merid, order;

	$this->isValid = false;
	preg_match($time_re, $str, $parts);
	if (count($parts) < 4) {
	    return false;
	}

	switch ($this->dateOrder) {
	case 'dmy':
	case 'mdy':
	case 'ymd':
	    $order = $this->dateOrder;
	    break;
	default:
	    # Auto detect, initially based on the date separator.
	    switch ($parts[2]) {
	    case '/':	# US-style m/d/y
		$order = 'mdy';
		break;
	    case '.':	# Euro-style d.m.y
		$order = 'dmy';
		break;
	    default:	# ISO-style y-m-d
		$order = 'ymd';
		break;
	    }

	    # Magnitudes may also dictate order.
	    if ($parts[1] > 31) {
		# The year is first.
		$order = 'ymd';
	    } else if ($parts[1] > 12) {
		# The day is first.
		$order = 'dmy';
	    } else if ($parts[3] > 12) {
		# The day is second.
		$order = 'mdy';
	    } else if ($parts[4] > 31 && $order == 'ymd') {
		# The year is last, not first. Take our best guess.
		$order = 'dmy';
	    }
	    break;
	}

	switch ($order) {
	case 'dmy':
	    $day = $parts[1]; $month = $parts[3]; $year = $parts[4];
	    break;
	case 'mdy':
	    $month = $parts[1]; $day = $parts[3]; $year = $parts[4];
	    break;
	case 'ymd':
	    $year = $parts[1]; $month = $parts[3]; $day = $parts[4];
	    break;
	}

	$hour = intval($parts[5]); $minute = intval($parts[6]);
	if ($hour > 99) { $minute = $hour % 100; $hour = floor($hour / 100); }
	$merid = $parts[7];
	switch ($merid) {
	case 'a':
	case 'A':
	    if ($hour == 12) { $hour = 0; }
	    break;
	case 'p':
	case 'P':
	    if ($hour < 12) { $hour += 12; }
	    break;
	}

	if ($year >= 1000 && $year <= 9999 && $month > 0 && $month < 13 &&
	  $day > 0 && $day < 32 && $hour >= 0 && $hour < 25 &&
	  $minute >= 0 && $minute < 61) {
	    $this->year = $year; $this->month = $month; $this->day = $day;
	    $this->hour = $hour; $this->minute = $minute;
	    $this->isValid = true;
	}
	return $this->isValid;
    }

    # Parse a string already (more or less) in ISO 8601
    function parseIso8601 ($str) {
	$time_re = '/^(\d{4})\D*(\d{2})\D*(\d{2})(?:\D*(\d{2}))?(?:\D*(\d{2}))?$/';

	$this->isValid = false;
	preg_match($time_re, $str, $parts);
	if (count($parts) < 4) {
	    return false;
	}

	list($unused, $year, $month, $day, $hour, $minute) = $parts;
	$hour = (int) $hour; $minute = (int) $minute;
	if ($year >= 1000 && $year <= 9999 && $month > 0 && $month < 13 &&
	  $day > 0 && $day < 32 && $hour >= 0 && $hour < 25 &&
	  $minute >= 0 && $minute < 61) {
	    $this->year = $year; $this->month = $month; $this->day = $day;
	    $this->hour = $hour; $this->minute = $minute;
	    $this->isValid = true;
	}

	return $this->isValid;
    }

    # Return the parsed string in ISO 8601 format
    function toString () {
	return $this->isValid?
	  sprintf("%d-%02d-%02dT%02d:%02d", $this->year, $this->month,
	  $this->day, $this->hour, $this->minute): '';
    }

    function __toString () {
	return $this->toString();
    }

    # One-step parse and convert when called like a function (>= PHP 5.3.0)
    function __invoke ($str) {
	$this->parse($str);
	return $this->toString();
    }

}

# END
